@@ -1,13 +1,14 @@
 #!/usr/bin/env python
-__author__ = ('Aitor Blanco Miguez (aitor.blancomiguez@unitn.it), '
-              'Duy Tin Truong (duytin.truong@unitn.it), '
-              'Francesco Asnicar (f.asnicar@unitn.it), '
-              'Moreno Zolfo (moreno.zolfo@unitn.it), '
-              'Francesco Beghini (francesco.beghini@unitn.it),'
-              'Michal Puncochar (michal.puncochar@unitn.it')
-__version__ = '4.1.2'
-__date__ = '16 Oct 2025'
-
+__author__ = (
+    'Aitor Blanco Miguez (aitor.blancomiguez@unitn.it), '
+    'Duy Tin Truong (duytin.truong@unitn.it), '
+    'Francesco Asnicar (f.asnicar@unitn.it), '
+    'Moreno Zolfo (moreno.zolfo@unitn.it), '
+    'Francesco Beghini (francesco.beghini@unitn.it), '
+    'Michal Puncochar (michal.puncochar@unitn.it)'
+)
+__version__ = '4.2.4'
+__date__    = '21 Oct 2025'
 
 import argparse as ap
 import io
@@ -15,7 +16,6 @@ import os
 import re
 import tempfile
 import time
-from collections import OrderedDict
 from shutil import copyfile, rmtree
 from typing import Iterable
 
@@ -49,28 +49,35 @@ class Strainphlan:
         """
 
         return execute_pool(((Strainphlan.get_matrix_for_sample, sample_path, self.clade_markers_names,
-                              self.breadth_thres) for sample_path in self.samples), self.nprocs)
+                              self.breadth_thres, self.polymorphism_perc, self.database_controller.get_database_name())
+                             for sample_path in self.samples), self.nprocs)
 
 
     @classmethod
-    def get_matrix_for_sample(cls, sample_path, clade_markers, breadth_thres):
+    def get_matrix_for_sample(cls, sample_path, clade_markers, breadth_thres, polymorphism_perc, database_name):
         """Returns the matrix with the presence / absence of the clade markers in a samples
 
         Args:
             sample_path (str): the path to the sample
             clade_markers (Iterable): the list with the clade markers names
             breadth_thres:
+            polymorphism_perc:
+            database_name:
 
         Returns:
             dict: dictionary containing the sample-to-markers information as a binary matrix
         """
         sample = ConsensusMarkers.from_file(sample_path)
+        if sample.database_name is not None and sample.database_name != database_name:
+            error(f'The database of the sample {sample.database_name} does not match the provided {database_name}',
+                  exit=True)
         sample_name = cls.sample_path_to_name(sample_path)
-
         markers = {"sample_name": sample_name}
         markers.update({m: 0 for m in clade_markers})
         markers.update({marker.name: 1 for marker in sample.consensus_markers
-                        if marker.name in clade_markers and marker.breadth >= breadth_thres})
+                        if marker.name in clade_markers and 
+                        marker.breadth >= breadth_thres and
+                        marker.get_polymorphism_perc() <= polymorphism_perc})
         return markers
 
 
@@ -603,7 +610,7 @@ class Strainphlan:
 
     def __init__(self, args):
         self.clade_markers_names = None
-        self.database_controller = MetaphlanDatabaseController(args.database)
+        self.database_controller = StrainphlanDatabaseController(args.database)
         self.clade_markers_file = args.clade_markers
         self.samples = Strainphlan.get_input_samples(args.samples)
         self.references = Strainphlan.get_input_samples(args.references)
@@ -616,6 +623,7 @@ class Strainphlan:
         self.sample_with_n_markers_after_filt = args.sample_with_n_markers_after_filt
         self.sample_with_n_markers_after_filt_perc = args.sample_with_n_markers_after_filt_perc
         self.breadth_thres = args.breadth_thres
+        self.polymorphism_perc = args.polymorphism_perc if args.polymorphism_perc is not None else 100
         self.print_clades_only = args.print_clades_only
         self.non_interactive = args.non_interactive
         self.phylophlan_mode = args.phylophlan_mode
@@ -666,6 +674,8 @@ def read_params():
                         "This rule is combined with AND logic with --sample_with_n_markers_after_filt")
     p.add_argument('--breadth_thres', type=int, default=80,
                    help="Threshold defining the minimum breadth of coverage for the markers")
+    p.add_argument('--polymorphism_perc', type=int, default=None,
+                   help="Threshold defining the maximum percentage of polymorphic sites in a marker to be considered ")
     p.add_argument('--phylophlan_mode', choices=['accurate', 'fast'], default='fast',
                    help="The presets for fast or accurate phylogenetic analysis")
     p.add_argument('--phylophlan_configuration', type=str, default=None,
