@@ -56,7 +56,33 @@ def fopen(fn):
     return open(fn)
 
 
-def read_and_write_raw_int(fd, min_len=None, prefix_id=""):
+### ADDED SPLIT READS OPTION
+def write_split_overlap(description, sequence, qual, min_len, readlen, nsplits, avg_read_length, prefix_id, fmt, idx=1):
+    description = clean_read_id(description, forced=True)
+    i = 0
+    while len(sequence) > min_len:
+        cur = sequence[:readlen]
+        if qual:
+            cur_qual = qual[:readlen]
+        else:
+            cur_qual = None
+        avg_read_length = len(cur) + avg_read_length
+        _ = sys.stdout.write(print_record(description + "__{}{}{}".format(prefix_id, '.' if prefix_id else '', idx) + "_{}".format(i), cur, cur_qual, fmt))
+        
+        ## ADD OVERLAP
+        sequence = sequence[(readlen - (readlen//5)):]
+        if qual:
+            qual = qual[(readlen - (readlen//5)):]
+
+        ## NO OVERLAP
+        # sequence = sequence[readlen:]
+        # qual = qual[readlen:]
+        i += 1
+    nsplits += i
+    return (nsplits, avg_read_length)
+
+
+def read_and_write_raw_int(fd, min_len=None, prefix_id="", split_reads=None):
     fmt = None
     avg_read_length = 0
     nreads = 0
@@ -87,12 +113,16 @@ def read_and_write_raw_int(fd, min_len=None, prefix_id=""):
             qual = None
             description, sequence = record
 
-        if len(sequence) >= min_len:
-            description = clean_read_id(description, forced=True)
-            avg_read_length = len(sequence) + avg_read_length
-            _ = sys.stdout.write(print_record(description + "__{}{}1".format(prefix_id, '.' if prefix_id else ''), sequence, qual, fmt))
+        if split_reads:
+            nreads, avg_read_length = write_split_overlap(description, sequence, qual, min_len, split_reads, nreads, avg_read_length, prefix_id, fmt)
+        
         else:
-            discarded = discarded + 1
+            if len(sequence) >= min_len:
+                description = clean_read_id(description, forced=True)
+                avg_read_length = len(sequence) + avg_read_length
+                _ = sys.stdout.write(print_record(description + "__{}{}1".format(prefix_id, '.' if prefix_id else ''), sequence, qual, fmt))
+            else:
+                discarded = discarded + 1
 
     # parse and check all the remaining reads
     for idx, record in enumerate(parser(fd), 2):
@@ -101,14 +131,17 @@ def read_and_write_raw_int(fd, min_len=None, prefix_id=""):
         else:
             qual = None
             description, sequence = record
-
-        if len(sequence) >= min_len:
-            avg_read_length = len(sequence) + avg_read_length
-            description = clean_read_id(description, forced=True)
-            _ = sys.stdout.write(
-                    print_record(description + "__{}{}{}".format(prefix_id, '.' if prefix_id else '', idx), sequence, qual, fmt))
+        
+        if split_reads:
+            nreads, avg_read_length = write_split_overlap(description, sequence, qual, min_len, split_reads, nreads, avg_read_length, prefix_id, fmt, idx)
+        
         else:
-            discarded = discarded + 1
+            if len(sequence) >= min_len:
+                avg_read_length = len(sequence) + avg_read_length
+                description = clean_read_id(description, forced=True)
+                _ = sys.stdout.write(print_record(description + "__{}{}{}".format(prefix_id, '.' if prefix_id else '', idx), sequence, qual, fmt))
+            else:
+                discarded = discarded + 1
     # else:
     #     for idx, l in enumerate(fd,1):
     #         avg_read_length = len(l) + avg_read_length
@@ -118,7 +151,8 @@ def read_and_write_raw_int(fd, min_len=None, prefix_id=""):
         sys.stderr.write('Error: no reads found.\n')
         sys.exit(1)
 
-    nreads = idx - discarded
+    if not split_reads:
+        nreads = idx - discarded
 
     if not nreads:
         sys.stderr.write('Error: no reads longer than {} bp found.\n'.format(min_len))
@@ -127,12 +161,12 @@ def read_and_write_raw_int(fd, min_len=None, prefix_id=""):
     return (nreads, avg_read_length)
 
 
-def read_and_write_raw(fd, opened=False, min_len=None, prefix_id=""):
+def read_and_write_raw(fd, opened=False, min_len=None, prefix_id="", split_reads=None):
     if opened:  # fd is stdin
-        nreads, avg_read_length = read_and_write_raw_int(fd, min_len=min_len, prefix_id=prefix_id)
+        nreads, avg_read_length = read_and_write_raw_int(fd, min_len=min_len, prefix_id=prefix_id, split_reads=split_reads)
     else:
         with fopen(fd) as inf:
-            nreads, avg_read_length = read_and_write_raw_int(inf, min_len=min_len, prefix_id=prefix_id)
+            nreads, avg_read_length = read_and_write_raw_int(inf, min_len=min_len, prefix_id=prefix_id, split_reads=split_reads)
 
     return (nreads, avg_read_length)
 
@@ -142,6 +176,7 @@ def main():
     args = []
     nreads = None
     avg_read_length = None
+    split_reads = None
 
     if len(sys.argv) > 1:
         for l in sys.argv[1:]:
@@ -153,11 +188,16 @@ def main():
                 min_len = int(l)
             elif l in ['-l', '--min_len']:
                 min_len = 'next'
+            ### ADDED SPLIT READS OPTION
+            elif split_reads == 'next':
+                split_reads = int(l) if int(l)!=0 else None
+            elif l == '--split_reads':
+                split_reads = 'next'
             else:
                 args.append(l)
 
     if len(args) == 0:
-        nreads, avg_read_length = read_and_write_raw(sys.stdin, opened=True, min_len=min_len)
+        nreads, avg_read_length = read_and_write_raw(sys.stdin, opened=True, min_len=min_len, split_reads=split_reads)
     else:
         files = []
         avg_read_length, nreads = 0, 0
@@ -170,18 +210,18 @@ def main():
                     files += [f]
 
         for prefix_id, f in enumerate(files, 1):
-            f_nreads, f_avg_read_length = read_and_write_raw(f, opened=False, min_len=min_len, prefix_id=prefix_id)
+            f_nreads, f_avg_read_length = read_and_write_raw(f, opened=False, min_len=min_len, prefix_id=prefix_id, split_reads=split_reads)
             nreads += f_nreads
             avg_read_length += f_avg_read_length
 
+    nbases = avg_read_length
     avg_read_length /= nreads
 
     if nreads and avg_read_length:
-        sys.stderr.write('{}\t{}'.format(nreads, avg_read_length))
+        sys.stderr.write('{}\t{}\t{}'.format(nreads, avg_read_length, nbases))
     else:
         exit(1)
 
 
 if __name__ == '__main__':
     main()
-
