@@ -1,14 +1,14 @@
 #!/usr/bin/env python
-__author__ = ('Michal Puncochar (michal.puncochar@unitn.it), '
-              'Aitor Blanco Miguez (aitor.blancomiguez@unitn.it), '
-              'Duy Tin Truong (duytin.truong@unitn.it), '
-              'Francesco Asnicar (f.asnicar@unitn.it), '
-              'Moreno Zolfo (moreno.zolfo@unitn.it), '
-              'Francesco Beghini (francesco.beghini@unitn.it)')
-__version__ = '4.1.2'
-__date__ = '16 Oct 2025'
-
-
+__author__ = (
+    'Michal Puncochar (michal.puncochar@unitn.it), '
+    'Aitor Blanco Miguez (aitor.blancomiguez@unitn.it), '
+    'Duy Tin Truong (duytin.truong@unitn.it), '
+    'Francesco Asnicar (f.asnicar@unitn.it), '
+    'Moreno Zolfo (moreno.zolfo@unitn.it), '
+    'Francesco Beghini (francesco.beghini@unitn.it)'
+)
+__version__ = '4.2.4'
+__date__    = '21 Oct 2025'
 
 import argparse as ap
 import bz2
@@ -16,8 +16,11 @@ import os
 import subprocess as sp
 import tempfile
 import time
+import itertools as it
 from collections import Counter
+from dataclasses import dataclass
 from shutil import rmtree
+from typing import TextIO
 
 import numpy as np
 import pysam
@@ -27,27 +30,83 @@ try:
     from .external_exec import samtools_sam_to_bam, samtools_sort_bam_v1, decompress_bz2
     from .util_fun import info, error, warning
     from .parallelisation import execute_pool
-    from .database_controller import MetaphlanDatabaseController
+    from .database_controller import StrainphlanDatabaseController
     from .consensus_markers import ConsensusMarker, ConsensusMarkers
 except ImportError:
     from external_exec import samtools_sam_to_bam, samtools_sort_bam_v1, decompress_bz2
     from util_fun import info, error, warning
     from parallelisation import execute_pool
-    from database_controller import MetaphlanDatabaseController
+    from database_controller import StrainphlanDatabaseController
     from consensus_markers import ConsensusMarker, ConsensusMarkers
 
 
+@dataclass
+class MapperSpecificDefaults:
+    min_reads_aligning: int
+    min_mapping_quality: int
+
+
+class GenericSamFilter:
+    def __init__(self, min_mapping_quality, markers_subset):
+        self.min_mapping_quality = min_mapping_quality
+        self.markers_subset = markers_subset
+
+
+    def filter_mapping_line(self, aln):
+        marker = aln.reference_name
+
+        if marker.startswith('VDB'):
+            return False
+
+        if self.markers_subset is not None and marker not in self.markers_subset:
+            return False
+
+        if aln.is_secondary or aln.is_qcfail or aln.is_unmapped:
+            return False
+
+        if aln.mapping_quality < self.min_mapping_quality:
+            return False
+
+        return True
+
+
+class Bowtie2SamFilter(GenericSamFilter):
+    pass
+
+
+class MinimapSamFilter(GenericSamFilter):
+    def __init__(self, min_mapping_quality, markers_subset, max_gcsd):
+        super().__init__(min_mapping_quality, markers_subset)
+        self.max_gcsd = max_gcsd
+
+    def filter_mapping_line(self, aln):
+        if not super().filter_mapping_line(aln):
+            return False
+
+        tags = dict(aln.tags)
+        gcsd = tags['de']
+
+        if gcsd > self.max_gcsd:
+            return False
+
+        return True
+
+
+
 class SampleToMarkers:
-    """SampleToMarkers class"""
+    MAPPER_SPECIFIC_DEFAULTS = {
+        'bowtie2': MapperSpecificDefaults(min_reads_aligning=8, min_mapping_quality=10),
+        'minimap2': MapperSpecificDefaults(min_reads_aligning=1, min_mapping_quality=50),
+        None: MapperSpecificDefaults(min_reads_aligning=1, min_mapping_quality=0),
+    }
 
     class DEFAULTS:
-        min_reads_aligning = 8
         depth_avg_q = 0.2
         quasi_marker_frac = 0.33
         min_breadth = 80
         min_base_coverage = 1
         min_base_quality = 30
-        min_mapping_quality = 10
+        max_gcsd = 0.1
         poly_dominant_frq_thrsh = 0.8
 
     class CONSTANTS:
@@ -240,9 +299,9 @@ class SampleToMarkers:
         return consensus_markers_filtered
 
 
-    @staticmethod
-    def parallel_filter_sam(input_file, tmp_dir, input_format, min_mapping_quality, min_reads_aligning,
-                            filtered_markers, all_markers):
+    @classmethod
+    def parallel_filter_sam(cls, input_file, tmp_dir, input_format, min_mapping_quality, min_reads_aligning,
+                            max_gcsd, filtered_markers, all_markers, db_name):
         """
         Filters an input SAM file
             * filters out viral markers (VDB)
@@ -255,72 +314,118 @@ class SampleToMarkers:
             input_format:
             min_reads_aligning:
             min_mapping_quality:
-            filtered_markers (set): the list with the markers of the filtered clades, None if to use all markers
+            max_gcsd:
+            filtered_markers (set): the set with the markers of the filtered clades, None if to use all markers
             all_markers:
+            db_name:
 
         Returns:
             str: the path to the output file
         """
         output_file = os.path.join(tmp_dir, input_file.split('/')[-1])
+        ifn: TextIO
         if input_format.lower() == "bz2":
-            ifn = bz2.open(input_file, 'rb')
+            ifn = bz2.open(input_file, 'rt')
             output_file = output_file.replace('.bz2', '')
         else:
             assert input_format.lower() == "sam"
-            ifn = open(input_file, 'rb')
+            ifn = open(input_file, 'rt')
 
-        def filter_mapping_line(line_fields_, markers_subset):
-            marker_ = line_fields_[2]
-            if marker_.startswith(b'VDB'):
-                return False
-            if markers_subset is not None and marker_.decode() not in markers_subset:
-                return False
-            mapq_ = int(line_fields_[4].decode())
-            if mapq_ < min_mapping_quality:
-                return False
-            return True
-
-        marker_to_reads = Counter()
+        # Pass through header
+        mapper_name = None
+        first_aln_line = None
+        header_lines = []
         for line in ifn:
-            if line.startswith(b'@'):
-                continue
-            line_fields = line.rstrip(b'\n').split(b'\t')
-            if filter_mapping_line(line_fields, filtered_markers):
-                marker = line_fields[2].decode()
-                marker_to_reads[marker] += 1
-                if marker not in all_markers:
-                    error(f'Marker {marker} not in the metaphlan database', exit=True)
+            if line.startswith('@'):
+                if line.startswith('@CO'):
+                    for item in line[len('@CO'):].strip().split('\t'):
+                        if item.startswith('index:'):
+                            db_name_sam = item[len('index:'):].strip().strip('"')
+                            if db_name_sam != db_name:
+                                error(f'The database of the sample {db_name_sam} does not match {db_name}', exit=True)
+                elif line.startswith('@PG'):
+                    for item in line[len('@PG'):].strip().split('\t'):
+                        if item.startswith('PN:'):
+                            mapper_name = item[len('PN:'):].strip().strip('"')
+                header_lines.append(line)
+            else:  # already in the alignment part
+                first_aln_line = line
+                break
+
+
+        # Resolve mapper specific default arguments and sam filtering
+        if min_reads_aligning is None or min_mapping_quality is None:
+            if mapper_name is None:
+                warning('Could not infer the mapper from the SAM file header, using default parameters')
+            elif mapper_name not in cls.MAPPER_SPECIFIC_DEFAULTS:
+                warning(f'The mapper name {mapper_name} is not recognized, using default parameters')
+                mapper_name = None
+            else:
+                info(f'Setting default parameters for mapper {mapper_name}')
+
+            if min_reads_aligning is None:
+                min_reads_aligning = cls.MAPPER_SPECIFIC_DEFAULTS[mapper_name].min_reads_aligning
+            if min_mapping_quality is None:
+                min_mapping_quality = cls.MAPPER_SPECIFIC_DEFAULTS[mapper_name].min_mapping_quality
+
+        if mapper_name == 'bowtie2':
+            sam_filter = Bowtie2SamFilter(min_mapping_quality, filtered_markers)
+        elif mapper_name == 'minimap2':
+            sam_filter = MinimapSamFilter(min_mapping_quality, filtered_markers, max_gcsd)
+        else:
+            sam_filter = GenericSamFilter(min_mapping_quality, filtered_markers)
+
+
+        # Pass through alignment lines just to count hits
+        header = pysam.AlignmentHeader.from_text(''.join(header_lines))
+        marker_to_reads = Counter()
+        if first_aln_line is not None:
+            for line in it.chain([first_aln_line], ifn):
+                aln = pysam.AlignedSegment.fromstring(line, header)
+                if sam_filter.filter_mapping_line(aln):
+                    marker = aln.reference_name
+                    marker_to_reads[marker] += 1
+                    if marker not in all_markers:
+                        error(f'Marker {marker} not in the metaphlan database', exit=True)
 
         selected_markers = set((m for m, c in marker_to_reads.items() if c >= min_reads_aligning))
+        sam_filter.markers_subset = selected_markers
 
-        ifn.seek(0)  # second pass of the file
-        with open(output_file, 'wb') as ofn:
+
+        # Second pass of the file to write the filtered output SAM file
+        ifn.seek(0)
+        with open(output_file, 'wt') as ofn:
             for line in ifn:
-                line_fields = line.rstrip(b'\n').split(b'\t')
-                if line.startswith(b'@SQ'):
-                    assert line_fields[1].startswith(b'SN:')  # in bowtie2 output SN is always the first tag
-                    marker = line_fields[1][3:].decode()
+                line_fields = line.rstrip('\n').split('\t')
+                if line.startswith('@SQ'):
+                    assert line_fields[1].startswith('SN:')  # in bowtie2 output SN is always the first tag
+                    marker = line_fields[1][3:]
                     if marker in selected_markers:
                         ofn.write(line)
-                elif line.startswith(b'@'):  # other header lines like @HD and @PG
+                elif line.startswith('@'):  # other header lines like @HD and @PG
                     ofn.write(line)
                 else:  # mapping lines
-                    if filter_mapping_line(line_fields, selected_markers):
+                    aln = pysam.AlignedSegment.fromstring(line, header)
+                    if sam_filter.filter_mapping_line(aln):
                         ofn.write(line)
 
         ifn.close()
         return output_file
 
+
     def filter_sam_files(self):
         """Filters the input SAM files with the hits against markers of specific clades and low quality reads"""
         filtered_markers = self.database_controller.get_filtered_markers(self.clades) if len(self.clades) > 0 else None
         all_markers = set(self.database_controller.get_all_markers())
+        db_name = self.database_controller.get_database_name()
         self.input = execute_pool(((SampleToMarkers.parallel_filter_sam, i, self.tmp_dir, self.input_format,
-                                    self.min_mapping_quality, self.min_reads_aligning, filtered_markers, all_markers)
+                                    self.min_mapping_quality, self.min_reads_aligning, self.max_gcsd, filtered_markers,
+                                    all_markers, db_name)
                                    for i in self.input), self.nprocs)
         self.input_format = 'sam'
         self.sorted = False
 
+
     def run_sample2markers(self):
         """Runs the full sample2markes pipeline"""
         info("Creating temporary directory...")
@@ -345,13 +450,14 @@ class SampleToMarkers:
     def __init__(self, args):
         self.input = args.input
         self.output_dir = args.output_dir
-        self.database_controller = MetaphlanDatabaseController(args.database)
+        self.database_controller = StrainphlanDatabaseController(args.database)
         self.tmp_dir = args.output_dir if args.tmp is None else args.tmp
         self.breadth_threshold = args.breadth_threshold
         self.input_format = args.input_format
         self.clades = args.clades
         self.sorted = args.sorted
         self.min_reads_aligning = args.min_reads_aligning
+        self.max_gcsd = args.max_gcsd
         self.min_base_coverage = args.min_base_coverage
         self.min_base_quality = args.min_base_quality
         self.min_mapping_quality = args.min_mapping_quality
@@ -368,7 +474,7 @@ def read_params():
     Returns:
         namespace: The populated namespace with the command line arguments
     """
-    p = ap.ArgumentParser(description="", formatter_class=ap.ArgumentDefaultsHelpFormatter)
+    p = ap.ArgumentParser(description="", formatter_class=ap.ArgumentDefaultsHelpFormatter, add_help=False)
     # required
     p.add_argument('-i', '--input', type=str, nargs='+', required=True, help="The input samples as SAM or BAM files")
     p.add_argument('-o', '--output_dir', type=str, required=True, help="The output directory")
@@ -380,14 +486,18 @@ def read_params():
                    help="Restricts the reconstruction of the markers to the specified clades")
     p.add_argument('-f', '--input_format', type=str, default="bz2", help="The input samples format {bam, sam, bz2}")
     p.add_argument('--sorted', action='store_true', default=False, help="Whether the BAM input files are sorted")
-    p.add_argument('--min_reads_aligning', type=int, default=SampleToMarkers.DEFAULTS.min_reads_aligning,
-                   help="The minimum number of reads to cover a marker")
+    p.add_argument('--min_reads_aligning', type=int, default=None,
+                   help="The minimum number of reads to cover a marker."
+                        "Default 8 for bowtie2, 1 for minimap2 nad 1 otherwise.")
     p.add_argument('--min_base_coverage', type=int, default=SampleToMarkers.DEFAULTS.min_base_coverage,
                    help="The minimum depth of coverage for a base to be considered")
     p.add_argument('--min_base_quality', type=int, default=SampleToMarkers.DEFAULTS.min_base_quality,
                    help="The minimum quality for a base to be considered. This is performed BEFORE --min_base_coverage")
-    p.add_argument('--min_mapping_quality', type=int, default=SampleToMarkers.DEFAULTS.min_mapping_quality,
-                   help="The minimum quality for a mapping of the read to be considered.")
+    p.add_argument('--min_mapping_quality', type=int, default=None,
+                   help="The minimum quality for a mapping of the read to be considered. "
+                        "Default 10 for bowtie2, 50 for minimap2 and 0 otherwise.")
+    p.add_argument('--max_gcsd', type=float, default=SampleToMarkers.DEFAULTS.max_gcsd,
+                   help="The maximum gap-compressed sequence divergence threshold to use in case of Minimap2 mapper.")
     p.add_argument('--dominant_frq_threshold', type=float, default=SampleToMarkers.DEFAULTS.poly_dominant_frq_thrsh,
                    help="The cutoff for degree of 'allele dominance' for a position to be considered polymorphic")
     p.add_argument('--quasi_marker_frac', type=float, default=SampleToMarkers.DEFAULTS.quasi_marker_frac,
@@ -403,6 +513,9 @@ def read_params():
                    help="If specified, StrainPhlAn will not remove the temporary folder. "
                         "Not available with inputs in BAM format")
     p.add_argument('-n', '--nprocs', type=int, default=1, help="The number of threads to execute the script")
+    p.add_argument('-v', '--version', action='version',
+                   version=f"StrainPhlAn sample2markers version {__version__} ({__date__})")
+    p.add_argument('-h', '--help', action='help', help="Show help.")
 
     return p.parse_args()
 
