@@ -1,17 +1,18 @@
 #!/usr/bin/env python
 __author__ = 'Aitor Blanco (aitor.blancomiguez@unitn.it'
-__version__ = '4.1.2'
-__date__ = '16 Oct 2025'
+__version__ = '4.2.4'
+__date__    = '21 Oct 2025'
 
 import os
 import time
 import argparse as ap
 try:
     from .util_fun import info, error
-    from .database_controller import MetaphlanDatabaseController
 except ImportError:
     from util_fun import info, error
-    from database_controller import MetaphlanDatabaseController
+
+GTDB_ASSIGNMENT_FILE = os.path.join(os.path.dirname(os.path.abspath(
+    __file__)), "mpa_vJan25_CHOCOPhlAnSGB_202503_SGB2GTDB.tsv")
 
 
 def read_params():
@@ -24,8 +25,6 @@ def read_params():
         description="", formatter_class=ap.ArgumentDefaultsHelpFormatter)
     p.add_argument('-i', '--input', type=str,
                    default=None, help="The input profile")
-    p.add_argument('-d', '--database', type=str, default='latest',
-                   help="The path to the MetaPhlAn PKL database")
     p.add_argument('-o', '--output', type=str,
                    default=None, help="The output profile")
     return p.parse_args()
@@ -43,7 +42,7 @@ def check_params(args):
         error('-o (or --output) must be specified', exit=True)
 
 
-def get_gtdb_profile(mpa_profile, gtdb_profile, database):
+def get_gtdb_profile(mpa_profile, gtdb_profile):
     """Creates the GDTB profile from a MPA one
 
     Args:
@@ -52,7 +51,7 @@ def get_gtdb_profile(mpa_profile, gtdb_profile, database):
     """
     tax_levels = ['d', 'p', 'c', 'o', 'f', 'g', 's']
     sgb2gtdb = dict()
-    with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), "{}_SGB2GTDB.tsv".format(database)), 'r') as read_file:
+    with open(GTDB_ASSIGNMENT_FILE, 'r') as read_file:
         for line in read_file:
             line = line.strip().split('\t')
             sgb2gtdb[line[0]] = line[1]
@@ -70,7 +69,7 @@ def get_gtdb_profile(mpa_profile, gtdb_profile, database):
                 elif 't__SGB' in line:
                     line = line.strip().split('\t')
                     gtdb_tax = sgb2gtdb[line[0].split('|')[-1][3:]]
-                    if gtdb_tax not in abundances['s']:
+                    if gtdb_tax not in abundances:
                         abundances['s'][gtdb_tax] = 0
                     abundances['s'][gtdb_tax] += float(line[2])
         tax_levels.reverse()
@@ -80,12 +79,11 @@ def get_gtdb_profile(mpa_profile, gtdb_profile, database):
                 new_level = tax_levels[tax_levels.index(tax_level)+1]
                 if new_tax not in abundances[new_level]:
                     abundances[new_level][new_tax] = 0
-                abundances[new_level][new_tax] += abundances[tax_level][tax]        
+                abundances[new_level][new_tax] += abundances[tax_level][tax]
         tax_levels.reverse()
-        total4level = {x:sum([y for x,y in abundances[x].items()]) for x in tax_levels}
         for tax_level in tax_levels:
             for tax in abundances[tax_level]:
-                wf.write('{}\t{}\n'.format(tax, abundances[tax_level][tax] * 100 / total4level[tax_level]))
+                wf.write('{}\t{}\n'.format(tax, abundances[tax_level][tax]))
 
 
 def main():
@@ -93,8 +91,7 @@ def main():
     args = read_params()
     info("Start execution")
     check_params(args)
-    database_controller = MetaphlanDatabaseController(args.database)
-    get_gtdb_profile(args.input, args.output, database_controller.get_database_name())
+    get_gtdb_profile(args.input, args.output)
     exec_time = time.time() - t0
     info("Finish execution ({} seconds)".format(round(exec_time, 2)))
 
